@@ -166,9 +166,8 @@
   stockpile/discard.  Returns {:info Path :command Path}."
   [cmdref stockpile dlo]
   (let [{:keys [path registry metrics]} dlo
-        {:keys [id received command version certname attempts]} cmdref
-        entry (stock/entry id (serialize-metadata
-                                received command version certname))
+        {:keys [id received command attempts]} cmdref
+        entry (stock/entry id (serialize-metadata received cmdref true))
         cmd-dest (.resolve path (entry-cmd-data-filename entry))]
     ;; We're going to assume that our moves will be atomic, and if
     ;; they're not, that we don't care about the possibility of
@@ -201,7 +200,13 @@
   ;; indicator that the unknown message may be complete.
   (let [{:keys [path registry metrics]} dlo
         digest (digest/sha1 [bytes])
-        metadata (serialize-metadata received "unknown" 0 digest)
+        metadata (serialize-metadata received
+                                     {:producer-ts nil
+                                      :command "unknown"
+                                      :version 0
+                                      :certname digest
+                                      :compression ""}
+                                     true)
         cmd-dest (.resolve path (str id \- metadata))]
     (Files/write cmd-dest bytes (oopts []))
     (let [info-dest (store-failed-command-info id metadata "unknown"
