@@ -10,7 +10,7 @@
             [clojure.string :as str]
             [puppetlabs.puppetdb.time :as pl-time]
             [puppetlabs.puppetdb.jdbc.internal :refer [limit-result-set!]]
-            [puppetlabs.puppetdb.schema :as pls]
+            [puppetlabs.puppetdb.schema :as pls :refer [defn-validated]]
             [schema.core :as s]
             [clojure.math.numeric-tower :as math]
             [puppetlabs.i18n.core :refer [trs]]))
@@ -538,6 +538,47 @@
   [^HikariConfig config metrics-registry]
   (.setMetricRegistry config metrics-registry))
 
+(defn block-on-schema-mismatch
+  "Compares the schema version the local PDB knows about against the version
+  in the database. If these versions don't match it indicates that either a
+  migration has been applied by another PDB or that the local PDB has been
+  upgraded before the needed migration has been applied. Raising an exception
+  here will cause all connection attempts from Hikari to fail until the local
+  PDB has been upgraded or a needed migration is applied"
+  [expected-schema]
+  {:pre [(integer? expected-schema)]}
+  (format
+   (str/join
+    " "
+    ["do $$"
+     "declare"
+     "  db_schema_version integer;"
+     "  expected_max integer = %d;"
+     "begin"
+     "  if exists (select from information_schema.tables"
+     "               where table_schema = 'public'"
+     "                 and table_name = 'schema_migrations')"
+     "  then"
+     "    select max(version) into db_schema_version from schema_migrations;"
+     "    case"
+     "      when db_schema_version > expected_max then"
+     "        raise exception"
+     "          'Please upgrade PuppetDB: your database contains schema
+                 migration %% which is too new for this version of PuppetDB.',
+                 db_schema_version;"
+     "      when db_schema_version < expected_max then"
+     "        raise exception"
+     "          'Please run PuppetDB with the migrate option set to true
+                 to upgrade your database. The detected migration level %% is
+                 out of date.', db_schema_version;"
+     "      else"
+     "        perform true;"  ; if schema versions match do nothing
+     "    end case;"
+     "  end if;"
+     "end;"
+     "$$ language plpgsql;"])
+   expected-schema))
+
 (defn make-connection-pool
   "Given a DB spec map containing :subprotocol, :subname, :user, and :password
   keys, return a pooled DB spec map (one containing just the :datasource key
@@ -552,7 +593,8 @@
             conn-lifetime
             read-only?
             pool-name
-            maximum-pool-size]
+            maximum-pool-size
+            expected-schema]
      :as db-spec}
     metrics-registry]
    (let [conn-lifetime-ms (some-> conn-max-age pl-time/to-millis)
@@ -562,7 +604,18 @@
        (.setJdbcUrl (str "jdbc:" subprotocol ":" subname))
        (.setAutoCommit false)
        (.setInitializationFailTimeout -1)
-       (.setTransactionIsolation "TRANSACTION_READ_COMMITTED"))
+       (.setTransactionIsolation "TRANSACTION_READ_COMMITTED")
+       ;; Because we currently disable autocommit and specify
+       ;; connectionInitSql, we need to set IsolateInternalQueries to
+       ;; true because otherwise hikaricp will run our connection init
+       ;; code which issues a select without committing it, which
+       ;; causes later attempts to (for example) change the isolation
+       ;; level to fail (because the pending transaction already
+       ;; includes that select).
+       (.setIsolateInternalQueries true))
+     (some->> (when expected-schema
+                (block-on-schema-mismatch expected-schema))
+              (.setConnectionInitSql config))
      (some->> pool-name (.setPoolName config))
      (some->> connection-timeout (.setConnectionTimeout config))
      (some->> maximum-pool-size (.setMaximumPoolSize config))
@@ -601,3 +654,55 @@
     (str "in ("
          (str/join "," (repeat (count coll) inner))
          ")")))
+
+(defn current-user []
+  (-> (query-to-vec "select user as user") first :user))
+
+(defn current-database []
+  (-> "select current_database();" query-to-vec first :current_database))
+
+(defn-validated has-database-privilege?
+  [user db privilege] :- s/Bool
+  (-> ["select has_database_privilege(?, ?, ?)" user db privilege]
+      query-to-vec first :has_database_privilege))
+
+(defn-validated has-role?
+  [user role privilege] :- s/Bool
+  (-> ["select pg_has_role(?, ?, ?)" user role privilege]
+      query-to-vec first :pg_has_role))
+
+(defn disconnect-db [db]
+  "Forcibly disconnects all connections to the named db.  Requires
+  that the current DB session has sufficient authorization."
+  (query-to-vec
+   (str "select pg_terminate_backend (pg_stat_activity.pid)"
+        "  from pg_stat_activity"
+        "  where pg_stat_activity.datname = ?"
+        "    and pid <> pg_backend_pid()")
+   db))
+
+(defn disconnect-db-role [db user]
+  "Forcibly disconnects all connections from the specified role to the
+  named db.  Requires that the current DB session has sufficient
+  authorization."
+  (query-to-vec
+   (str "select pg_terminate_backend (pg_stat_activity.pid)"
+        "  from pg_stat_activity"
+        "  where pg_stat_activity.datname = ?"
+        "    and pg_stat_activity.usename = ?"
+        "    and pid <> pg_backend_pid()")
+   db user))
+
+(defn revoke-role-db-access
+  [role db]
+  ;; revoke commands can't be parameterized with the pgjdbc driver right now
+  (do-commands
+   (format "revoke connect on database %s from %s restrict"
+           (double-quote db) (double-quote role))))
+
+(defn restore-role-db-access
+  [role db]
+  ;; grant commands can't be parameterized with the pgjdbc driver right now
+  (do-commands
+   (format "grant connect on database %s to %s"
+          (double-quote db) (double-quote role))))
